/***************************************************************************
 *   Copyright (C) 2008 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "core/partition.h"

#include "core/device.h"

#include "fs/filesystem.h"
#include "fs/filesystemfactory.h"

#include "util/externalcommand.h"

#include <QString>
#include <QStringList>
#include <QFileInfo>

#include <kdebug.h>
#include <klocale.h>

Partition::Partition(PartitionNode* parent, const Device& device, const PartitionRole& role, FileSystem* fs, qint64 sectorStart, qint64 sectorEnd, qint32 number, PartitionTable::Flags availableFlags, const QStringList& mountPoints, bool mounted, PartitionTable::Flags activeFlags, State state) :
	PartitionNode(),
	m_Number(number),
	m_Children(),
	m_Parent(parent),
	m_FileSystem(fs),
	m_Roles(role),
	m_FirstSector(sectorStart),
	m_LastSector(sectorEnd),
	m_DevicePath(device.deviceNode()),
	m_MountPoints(mountPoints),
	m_AvailableFlags(availableFlags),
	m_ActiveFlags(activeFlags),
	m_IsMounted(mounted),
	m_SectorSize(device.sectorSize()),
	m_State(state)
{
	Q_ASSERT(m_Parent);
}

Partition::~Partition()
{
	clearChildren();
	deleteFileSystem();
}

Partition::Partition(const Partition& other) :
	PartitionNode(),
	m_Number(other.m_Number),
	m_Children(),
	m_Parent(other.m_Parent),
	m_FileSystem(FileSystemFactory::create(other.fileSystem())),
	m_Roles(other.m_Roles),
	m_FirstSector(other.m_FirstSector),
	m_LastSector(other.m_LastSector),
	m_DevicePath(other.m_DevicePath),
	m_MountPoints(other.m_MountPoints),
	m_AvailableFlags(other.m_AvailableFlags),
	m_ActiveFlags(other.m_ActiveFlags),
	m_IsMounted(other.m_IsMounted),
	m_SectorSize(other.m_SectorSize),
	m_State(other.m_State)
{
	foreach(const Partition* child, other.children())
	{
		Partition* p = new Partition(*child);
		p->setParent(this);
		m_Children.append(p);
	}
}

Partition& Partition::operator=(const Partition& other)
{
	if (&other == this)
		return *this;

	clearChildren();

	foreach(const Partition* child, other.children())
	{
		Partition* p = new Partition(*child);
		p->setParent(this);
		m_Children.append(p);
	}

	m_Number = other.m_Number;
	m_FileSystem = FileSystemFactory::create(other.fileSystem());
	m_Roles = other.m_Roles;
	m_FirstSector = other.m_FirstSector;
	m_LastSector = other.m_LastSector;
	m_DevicePath = other.m_DevicePath;
	m_MountPoints = other.m_MountPoints;
	m_AvailableFlags = other.m_AvailableFlags;
	m_ActiveFlags = other.m_ActiveFlags;
	m_IsMounted = other.m_IsMounted;
	m_SectorSize = other.m_SectorSize;
	m_State = other.m_State;

	return *this;
}

QString Partition::deviceNode() const
{
	if (roles().has(PartitionRole::None) || roles().has(PartitionRole::Unallocated))
		return i18nc("@item partition name", "unallocated");

	if (state() == StateNew)
		return i18nc("@item partition name", "New Partition");

	if (state() == StateRestore)
		return i18nc("@item partition name", "Restored Partition");

	QString res = m_DevicePath + QString::number(number());

	if (state() == StateCopy)
		return i18nc("@item partition name", "Copy of %1", res);

	return res;
}

void Partition::move(qint64 newStartSector)
{
	const qint64 savedLength = length();
	setFirstSector(newStartSector);
	setLastSector(newStartSector + savedLength - 1);
}

qint64 Partition::sectorsUsed() const
{
	if (!roles().has(PartitionRole::Extended))
		return fileSystem().sectorsUsed();

	qint64 result = 0;
	foreach (const Partition* p, children())
		if (!p->roles().has(PartitionRole::Unallocated))
			result += p->length();

	return result;
}

qint64 Partition::minimumSectors() const
{
	 return fileSystem().minCapacity() / sectorSize();
}

qint64 Partition::maximumSectors() const
{
	 return fileSystem().maxCapacity() / sectorSize();
}

void Partition::adjustLogicalNumbers(qint32 deletedNumber, qint32 insertedNumber)
{
	if (!roles().has(PartitionRole::Extended))
		return;

	foreach (Partition* p, children())
	{
		if (deletedNumber > 4 && p->number() > deletedNumber)
			p->setNumber(p->number() - 1);
		else if (insertedNumber > 4 && p->number() >= insertedNumber)
			p->setNumber(p->number() + 1);
	}
}

qint64 Partition::maxFirstSector() const
{
	qint64 rval = -1;

	foreach (const Partition* child, children())
		if (!child->roles().has(PartitionRole::Unallocated) && (child->firstSector() < rval || rval == -1))
			rval = child->firstSector();

	return rval;
}

qint64 Partition::minLastSector() const
{
	qint64 rval = -1;

	foreach (const Partition* child, children())
		if (!child->roles().has(PartitionRole::Unallocated) && child->lastSector() > rval)
			rval = child->lastSector();

	return rval;
}

bool Partition::hasChildren() const
{
	foreach (const Partition* child, children())
		if (!child->roles().has(PartitionRole::Unallocated))
			return true;

	return false;
}

void Partition::checkChildrenMounted()
{
	bool isBusy = false;
	
	foreach (const Partition* child, children())
		if (child->isMounted())
		{
			isBusy = true;
			break;
		}

	setMounted(isBusy);
}

bool Partition::canMount() const
{
	// cannot mount if already mounted
	if (isMounted())
		return false;

	// cannot mount if we have no mount points
	return !mountPoints().isEmpty();
}

bool Partition::canUnmount() const
{
	return isMounted();
}

bool Partition::mount()
{
	if (isMounted())
		return false;

	bool success = false;
	foreach(const QString& mp, mountPoints())
	{
		ExternalCommand mountCmd("mount", QStringList() << deviceNode() << mp);
		if (mountCmd.run() && mountCmd.exitCode() == 0)
			success = true;
	}

	setMounted(success);

	return success;
}

bool Partition::unmount()
{
	if (!isMounted())
		return false;

	bool success = true;

	foreach(const QString& mp, mountPoints())
	{
		ExternalCommand umountCmd("umount", QStringList() << mp);
		if (!umountCmd.run() || umountCmd.exitCode() != 0)
			success = false;
	}

	setMounted(!success);

	return success;
}

void Partition::deleteFileSystem()
{
	delete m_FileSystem;
	m_FileSystem = NULL;
}

void Partition::setFileSystem(FileSystem* fs)
{
	m_FileSystem = fs;
}
