/***************************************************************************
 *   Copyright (C) 2008 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#if !defined(PARTITION__H)

#define PARTITION__H

#include "core/partitionnode.h"
#include "core/partitionrole.h"
#include "core/partitiontable.h"

#include <QList>
#include <QStringList>
#include <qglobal.h>

class Device;
class OperationStack;

class PartResizerWidget;
class InsertDialog;
class NewDialog;

class CreateFileSystemOperation;
class RestoreOperation;
class SetPartFlagsOperation;
class CopyOperation;
class NewOperation;
class ResizeOperation;

class SetPartGeometryJob;
class CreatePartitionJob;
class SetPartFlagsJob;
class RestoreFileSystemJob;

class FileSystem;

class QString;

class Partition : public PartitionNode
{
	friend class PartitionTable;
	friend class OperationStack;
	friend class Device;
	friend class PartitionNode;
	
	friend class PartResizerWidget;
	friend class InsertDialog;
	friend class NewDialog;

	friend class CreateFileSystemOperation;
	friend class RestoreOperation;
	friend class SetPartFlagsOperation;
	friend class CopyOperation;
	friend class NewOperation;
	friend class ResizeOperation;

	friend class SetPartGeometryJob;
	friend class CreatePartitionJob;
	friend class SetPartFlagsJob;
	friend class RestoreFileSystemJob;
	
	public:
		enum State
		{
			StateNone = 0,
			StateNew = 1,
			StateCopy = 2,
			StateRestore = 3
		};

	public:
		Partition(PartitionNode* parent, const Device& device, const PartitionRole& role, FileSystem* fs, qint64 sectorStart, qint64 sectorEnd, qint32 number, PartitionTable::Flags availableFlags = PartitionTable::FlagNone, const QStringList& mountPoints = QStringList(), bool mounted = false, PartitionTable::Flags activeFlags = PartitionTable::FlagNone, State state = StateNone);
		~Partition();

	public:
		Partition(const Partition&);
		Partition& operator=(const Partition&);

	public:
		qint32 number() const { return m_Number; }

		bool isRoot() const { return false; }

		PartitionNode* parent() { return m_Parent; }
		const PartitionNode* parent() const { return m_Parent; }

		Partitions& children() { return m_Children; }
		const Partitions& children() const { return m_Children; }
		
		const QString& devicePath() const { return m_DevicePath; }
		
		qint64 firstSector() const { return m_FirstSector; }
		qint64 lastSector() const { return m_LastSector; }
		qint64 sectorsUsed() const;
		qint32 sectorSize() const { return m_SectorSize; }
		qint64 length() const { return lastSector() - firstSector() + 1; }
		qint64 capacity() const { return length() * sectorSize(); }
		qint64 used() const { return sectorsUsed() < 0 ? -1 : sectorsUsed() * sectorSize(); }
		qint64 available() const { return sectorsUsed() < 0 ? -1 : capacity() - used(); }
		qint64 minimumSectors() const;
		qint64 maximumSectors() const;
		qint64 maxFirstSector() const;
		qint64 minLastSector() const;

		QString deviceNode() const;

		const PartitionRole& roles() const { return m_Roles; }

		const QStringList& mountPoints() const { return m_MountPoints; }
		
		PartitionTable::Flags activeFlags() const { return m_ActiveFlags; }
		PartitionTable::Flags availableFlags() const { return m_AvailableFlags; }

		bool isMounted() const { return m_IsMounted; }
		
		FileSystem& fileSystem() { return *m_FileSystem; }
		const FileSystem& fileSystem() const { return *m_FileSystem; }
		
		State state() const { return m_State; }
		bool hasChildren() const;

		bool mount();
		bool unmount();

		bool canMount() const;
		bool canUnmount() const;

		void adjustLogicalNumbers(qint32 deletedNumber, qint32 insertedNumber);
		void checkChildrenMounted();

		static qint64 minimumPartitionSectors() { return m_MinimumPartitionSectors; }
		
	protected:
		void append(Partition* p) { m_Children.append(p); }
		void setDevicePath(const QString& s) { m_DevicePath = s; }
		void setRoles(const PartitionRole& r) { m_Roles = r; }
		void setMountPoints(const QStringList& sl) { m_MountPoints = sl; }
		void setFlags(PartitionTable::Flags f) { m_ActiveFlags = f; }
		void setSectorSize(qint32 s) { m_SectorSize = s; }
		void setFirstSector(qint64 s) { m_FirstSector = s; }
		void setLastSector(qint64 s) { m_LastSector = s; }
		void move(qint64 newStartSector);
		void setMounted(bool b) { m_IsMounted = b; }
		void setFlag(PartitionTable::Flag f) { m_ActiveFlags |= f; }
		void unsetFlag(PartitionTable::Flag f) { m_ActiveFlags &= ~f; }
		void setParent(PartitionNode* p) { m_Parent = p; }
		void setFileSystem(FileSystem* fs);
		void setNumber(qint32 n) { m_Number = n; }
		void setState(State s) { m_State = s; }
		void deleteFileSystem();

	private:
		qint32 m_Number;
		Partitions m_Children;
		PartitionNode* m_Parent;
		FileSystem* m_FileSystem;
		PartitionRole m_Roles;
		qint64 m_FirstSector;
		qint64 m_LastSector;
		QString m_DevicePath;
		QStringList m_MountPoints;
		PartitionTable::Flags m_AvailableFlags;
		PartitionTable::Flags m_ActiveFlags;
		bool m_IsMounted;
		qint32 m_SectorSize;
		State m_State;

		static const qint64 m_MinimumPartitionSectors = 8192;
};

#endif

