/***************************************************************************
 *   Copyright (C) 2008 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "fs/filesystem.h"

#include "util/externalcommand.h"
#include "util/capacity.h"

#include <klocale.h>
#include <kdebug.h>

FileSystem::FileSystem(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& l, FileSystem::Type t) :
	m_Type(t),
	m_FirstSector(firstsector),
	m_LastSector(lastsector),
	m_SectorsUsed(sectorsused),
	m_Label(l)
{
}

qint64 FileSystem::minCapacity() const
{
	return 8 * Capacity::unitFactor(Capacity::Byte, Capacity::MiB);
}

qint64 FileSystem::maxCapacity() const
{
	return Capacity::unitFactor(Capacity::Byte, Capacity::EiB);
}

bool FileSystem::findExternal(const QString& cmdName, const QStringList& args, int expectedCode)
{
	ExternalCommand cmd(cmdName, args);
	if (!cmd.run())
		return false;

	return cmd.exitCode() == 0 || cmd.exitCode() == expectedCode;
}

QString FileSystem::name() const
{
	return nameForType(type());
}

static const QString* typeNames()
{
	// The KDE localization framework doesn't like this array if it's
	// just declared static outside any function. If we do it here,
	// it seems to work fine. That's why this is done so strangely here.
	static const QString s[] =
	{
		i18nc("@item/plain filesystem name", "unknown"),
		i18nc("@item/plain filesystem name", "extended"),

		i18nc("@item/plain filesystem name", "ext2"),
		i18nc("@item/plain filesystem name", "ext3"),
		i18nc("@item/plain filesystem name", "linuxswap"),
		i18nc("@item/plain filesystem name", "fat16"),
		i18nc("@item/plain filesystem name", "fat32"),
		i18nc("@item/plain filesystem name", "ntfs"),
		i18nc("@item/plain filesystem name", "reiser"),
		i18nc("@item/plain filesystem name", "reiser4"),
		i18nc("@item/plain filesystem name", "xfs"),
		i18nc("@item/plain filesystem name", "jfs"),
		i18nc("@item/plain filesystem name", "hfs"),
		i18nc("@item/plain filesystem name", "hfsplus"),
		i18nc("@item/plain filesystem name", "ufs"),
		i18nc("@item/plain filesystem name", "unformatted")
	};

	return s;
}
	
QString FileSystem::nameForType(FileSystem::Type t)
{
	Q_ASSERT(t >= 0);
	Q_ASSERT(t < __lastType);
	
	return typeNames()[t];
}

FileSystem::Type FileSystem::typeForName(const QString& s)
{
	for (quint32 i = 0; i < __lastType; i++)
		if (typeNames()[i] == s)
			return static_cast<FileSystem::Type>(i);

	return Unknown;
}

QList<FileSystem::Type> FileSystem::types()
{
	QList<FileSystem::Type> result;

	int i = Ext2; // first "real" filesystem
	while (i != __lastType)
		result.append(static_cast<FileSystem::Type>(i++));

	return result;
}

QString FileSystem::mountTitle() const
{
	return i18nc("@title:menu", "Mount");
}

QString FileSystem::unmountTitle() const
{
	return i18nc("@title:menu", "Unmount");
}

void FileSystem::move(qint64 newStartSector)
{
	const qint64 savedLength = length();
	setFirstSector(newStartSector);
	setLastSector(newStartSector + savedLength - 1);
}
